from __future__ import division
import hoomd
import hoomd.md
#import hoomd_script
from math import cos,sin

#####Hoomd Initialization#####
hoomd.context.initialize("--gpu_error_checking");
#####System and Particles#####
###Discs
#Center of the disc

#radius of the discs
length=20
radicilindre = 4
semilength = int(length/2)
radi = 1
#radius of the polymers
radi2=0.6
pos=[];types=[];masses=[];inertia=[];orient=[];
pos.append([0.0,0.0,0.0]);types.append('R');masses.append(1.0)
inertia.append([1.0/12.0*1.0*radi**2.0, 1.0/12.0*1.0*length**2.0 + 1.0/4.0*1.0*radi**2.0, 1.0/12.0*1.0*length**2.0 + 1.0/4.0*1.0*radi**2.0])
orient.append([1.0, 0, 0, 0]);

for i in range(1,13):
    for j in range(1,12):
        a=[1.5*j,1.5*i,6]
        pos.append(a);types.append('P'); masses.append(0.1); inertia.append([2.0/5.0*1.0*radi2**2.0,2.0/5.0*1.0*radi2**2.0,2.0/5.0*1.0*radi2**2.0]); orient.append([1.0, 0, 0, 0])
        b=[1.5*j,1.5*i,-6]
        pos.append(b);types.append('P'); masses.append(0.1); inertia.append([2.0/5.0*1.0*radi2**2.0,2.0/5.0*1.0*radi2**2.0,2.0/5.0*1.0*radi2**2.0]); orient.append([1.0, 0, 0, 0])
        c=[1.5*j,1.5*i,-8]
        pos.append(c);types.append('P'); masses.append(0.1); inertia.append([2.0/5.0*1.0*radi2**2.0,2.0/5.0*1.0*radi2**2.0,2.0/5.0*1.0*radi2**2.0]); orient.append([1.0, 0, 0, 0])
        d=[1.5*j,1.5*i,8]
        pos.append(d);types.append('P'); masses.append(0.1); inertia.append([2.0/5.0*1.0*radi2**2.0,2.0/5.0*1.0*radi2**2.0,2.0/5.0*1.0*radi2**2.0]); orient.append([1.0, 0, 0, 0])

uc = hoomd.lattice.unitcell(N = len(pos),
                            a1 = [28., 0,   0],
                            a2 = [0,    28., 0],
                            a3 = [0,    0,   28.],
                            dimensions = 3,
                            position = pos,
                            type_name = types,
                            mass = masses,
                            moment_inertia =inertia,
                            orientation = orient);

system = hoomd.init.create_lattice(unitcell=uc, n=[5,5,5]);
system.particles.types.add('A');
#system.particles.types.add('P');
#surrounding particles
#Definition of the composing particles positions:
p=[]

dl = 2*radi
Nl = int(float(semilength)/dl)

for i in [-Nl, Nl]:
    a = i*dl
    for j in range(-2,3):
        for k in range(-2,3):
            p.append((a,j,k))


for i in range(-Nl+1, Nl):
    a = i*dl
    for j in [-2,2]:
        for k in [-1,0,1]:
            p.append((a,j,k))
    for j in [-2,-1,0,1,2]:
        for k in [-2,2]:
            p.append((a,j,k))
rigid = hoomd.md.constrain.rigid();
rigid.set_param('R',
                types=['A']*len(p),
                positions=p);
rigid.create_bodies()


#####Definition of the Interactions#####
nl = hoomd.md.nlist.cell()
###Truncated Lennard-Jones interaction at its minimum
lj = hoomd.md.pair.lj(r_cut=2.0**(1.0/6.0), nlist=nl, name="lennard-jones")
lj.set_params(mode='shift')
#We consider discs of 1 micro-m of height, therefore, the points will have 0.5 m-m radium
lj.pair_coeff.set(['R', 'A'], ['R', 'A'], epsilon=1.0, sigma=2.*radi, r_cut=2.0**(1.0/6.0))
#For the polymers we do the same
lj.pair_coeff.set(['P'], ['R', 'A'], epsilon=1.0, sigma=radi+radi2,r_cut=2.0**(1.0/6.0)*(1.0/2+radi2))
lj.pair_coeff.set(['P'], ['P'], epsilon=1.0, sigma=2*radi2,r_cut=2.0**(1.0/6.0)*2*radi2)
#####Integrator#####
#hoomd.md.integrate.mode_standard(dt=0.0001);
rigidint = hoomd.group.rigid_center();
ljparts = hoomd.group.type(name='ljparts', type='P');
groupint = hoomd.group.union('partsint', rigidint, ljparts);
hoomd.md.integrate.langevin(group=groupint, kT=1., seed=354465);



#####Data Writting#####
hoomd.analyze.log(filename="log-output.log",
                  quantities=['potential_energy',
                              'translational_kinetic_energy',
                              'rotational_kinetic_energy',
			      'momentum',
			      'pressure',
			      'pair_lj_energy_lennard-jones'],
                  period=1000,
                  overwrite=True, phase=0);
gsd_restart=hoomd.dump.gsd(filename="restart.gsd",group=hoomd.group.all(), period=5e5, truncate=True, phase=0)
hoomd.dump.gsd("trajectory.gsd",period=5e5,group=hoomd.group.all(),overwrite=True, phase=0);
hoomd.dump.gsd("trajectory_discs.gsd",period=1e4,group=rigidint,overwrite=True, phase=0);


#####Simulation Running#####
t1=1e6
l1=28*4*5./3. # 
l2=12*4*5./3.
t2=2e7

#Disorder
hoomd.md.integrate.mode_standard(dt=0.000005);
#reduction to 3/4
hoomd.update.box_resize(Lx = hoomd.variant.linear_interp([(0, l1), (t1, l2)]),
                  Ly = hoomd.variant.linear_interp([(0, l1), (t1, l2)]),
                  Lz = hoomd.variant.linear_interp([(0, l1), (t1, l2)]))
hoomd.run(t1);

hoomd.md.integrate.mode_standard(dt=0.0001);
hoomd.run(t2);
